@@ -7,8 +7,7 @@ from pretend import stub
 from six import StringIO
 
 from tests.utils import DummyTransport, assert_nodes_equal
-from zeep import wsdl
-from zeep import Client
+from zeep import Client, wsdl
 from zeep.transports import Transport
 
 
@@ -36,6 +35,7 @@ def test_parse_soap_wsdl():
         m.post('http://example.com/stockquote', text=response)
         account_type = client.get_type('stoc:account')
         account = account_type(id=100)
+        account.user = 'mvantellingen'
         country = client.get_element('stoc:country').type()
         country.name = 'The Netherlands'
         country.code = 'NL'
@@ -58,7 +58,7 @@ def test_parse_soap_wsdl():
                 <tickerSymbol>foobar</tickerSymbol>
                 <account>
                   <id>100</id>
-                  <user/>
+                  <user>mvantellingen</user>
                 </account>
                 <stoc:country>
                   <name>The Netherlands</name>
@@ -100,7 +100,8 @@ def test_parse_soap_header_wsdl():
                 }
             })
 
-        assert result == 120.123
+        assert result.body.price == 120.123
+        assert result.header.body is None
 
         request = m.request_history[0]
 
@@ -324,10 +325,12 @@ def test_create_import_schema(recwarn):
     """.strip())
 
     transport = DummyTransport()
-    transport.bind('a.xsd', schema_node_a)
-    transport.bind('b.xsd', schema_node_b)
+    transport.bind('http://tests.python-zeep.org/a.xsd', schema_node_a)
+    transport.bind('http://tests.python-zeep.org/b.xsd', schema_node_b)
+
+    document = wsdl.Document(
+        content, transport, 'http://tests.python-zeep.org/content.wsdl')
 
-    document = wsdl.Document(content, transport)
     assert len(recwarn) == 0
     assert document.types.get_element('{http://tests.python-zeep.org/b}global')
 
@@ -366,10 +369,11 @@ def test_wsdl_imports_xsd(recwarn):
     """.strip())
 
     transport = DummyTransport()
-    transport.bind('a.xsd', schema_node_a)
-    transport.bind('b.xsd', schema_node_b)
+    transport.bind('http://tests.python-zeep.org/a.xsd', schema_node_a)
+    transport.bind('http://tests.python-zeep.org/b.xsd', schema_node_b)
 
-    wsdl.Document(content, transport)
+    wsdl.Document(
+        content, transport, 'http://tests.python-zeep.org/content.wsdl')
 
 
 def test_import_schema_without_location(recwarn):
@@ -445,10 +449,11 @@ def test_import_schema_without_location(recwarn):
     """.strip())
 
     transport = DummyTransport()
-    transport.bind('a.xsd', schema_node_a)
-    transport.bind('b.xsd', schema_node_b)
+    transport.bind('http://tests.python-zeep.org/a.xsd', schema_node_a)
+    transport.bind('http://tests.python-zeep.org/b.xsd', schema_node_b)
 
-    document = wsdl.Document(content, transport)
+    document = wsdl.Document(
+        content, transport, 'http://tests.python-zeep.org/content.wsdl')
     assert len(recwarn) == 0
     assert document.types.get_type('{http://tests.python-zeep.org/b}foo')
 
@@ -826,3 +831,73 @@ def test_parse_operation_empty_nodes():
     """.strip())
 
     assert wsdl.Document(content, None)
+
+
+def test_wsdl_duplicate_tns(recwarn):
+    wsdl_main = StringIO("""
+        <?xml version="1.0"?>
+        <wsdl:definitions
+          xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
+          xmlns:xsd="http://www.w3.org/2001/XMLSchema"
+          xmlns:tns="http://tests.python-zeep.org/xsd-main"
+          xmlns:sec="http://tests.python-zeep.org/wsdl-secondary"
+          xmlns:xsd-sec="http://tests.python-zeep.org/xsd-secondary"
+          xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/"
+          xmlns:wsdlsoap="http://schemas.xmlsoap.org/wsdl/soap/"
+          targetNamespace="http://tests.python-zeep.org/xsd-main">
+
+          <wsdl:import namespace="http://tests.python-zeep.org/xsd-main"
+            location="http://tests.python-zeep.org/schema-2.wsdl"/>
+
+
+          <wsdl:binding name="TestBinding" type="tns:TestPortType">
+            <soap:binding style="document" transport="http://schemas.xmlsoap.org/soap/http"/>
+            <wsdl:operation name="TestOperation1">
+              <soap:operation soapAction=""/>
+              <wsdl:input>
+                <soap:body use="literal"/>
+              </wsdl:input>
+            </wsdl:operation>
+          </wsdl:binding>
+          <wsdl:service name="TestService">
+            <wsdl:documentation>Test service</wsdl:documentation>
+            <wsdl:port name="TestPortType" binding="tns:TestBinding">
+              <soap:address location="http://tests.python-zeep.org/test"/>
+            </wsdl:port>
+          </wsdl:service>
+        </wsdl:definitions>
+    """.strip())
+
+    wsdl_2 = ("""
+        <?xml version="1.0"?>
+        <wsdl:definitions
+          xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
+          xmlns:xsd="http://www.w3.org/2001/XMLSchema"
+          xmlns:tns="http://tests.python-zeep.org/xsd-main"
+          xmlns:mine="http://tests.python-zeep.org/xsd-secondary"
+          xmlns:wsdlsoap="http://schemas.xmlsoap.org/wsdl/soap/"
+          targetNamespace="http://tests.python-zeep.org/xsd-main">
+
+          <wsdl:types>
+            <xsd:schema
+                targetNamespace="http://tests.python-zeep.org/xsd-main"
+                xmlns:tns="http://tests.python-zeep.org/xsd-main">
+              <xsd:element name="input" type="xsd:string"/>
+            </xsd:schema>
+          </wsdl:types>
+          <wsdl:message name="message-1">
+            <wsdl:part name="response" element="tns:input"/>
+          </wsdl:message>
+
+          <wsdl:portType name="TestPortType">
+            <wsdl:operation name="TestOperation1">
+              <wsdl:input message="message-1"/>
+            </wsdl:operation>
+          </wsdl:portType>
+        </wsdl:definitions>
+    """.strip())
+
+    transport = DummyTransport()
+    transport.bind('http://tests.python-zeep.org/schema-2.wsdl', wsdl_2)
+    document = wsdl.Document(wsdl_main, transport)
+    document.dump()
