@@ -3,7 +3,8 @@ import io
 import pytest
 from lxml import etree
 
-from tests.utils import DummyTransport, assert_nodes_equal, load_xml, render_node
+from tests.utils import (
+    DummyTransport, assert_nodes_equal, load_xml, render_node)
 from zeep import xsd
 
 
