@@ -1,7 +1,7 @@
 import pytest
 from lxml import etree
 
-from tests.utils import assert_nodes_equal, render_node, load_xml
+from tests.utils import assert_nodes_equal, load_xml, render_node
 from zeep import xsd
 
 
