@@ -1,11 +1,13 @@
+import copy
+
 import pytest
 from lxml import etree
 
-from tests.utils import assert_nodes_equal, load_xml
+from tests.utils import assert_nodes_equal, load_xml, render_node
 from zeep import xsd
 
 
-def test_complex_type_nested_wrong_type():
+def test_xml_complex_type_nested_wrong_type():
     schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
@@ -34,7 +36,7 @@ def test_complex_type_nested_wrong_type():
         container_elm(item={'bar': 1})
 
 
-def test_element_with_annotation():
+def test_xml_element_with_annotation():
     schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
@@ -59,7 +61,7 @@ def test_element_with_annotation():
     address_type(foo='bar')
 
 
-def test_complex_type_parsexml():
+def test_xml_complex_type_parsexml():
     schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
@@ -88,8 +90,8 @@ def test_complex_type_parsexml():
     assert obj.foo == 'bar'
 
 
-def test_array():
-    node = etree.fromstring("""
+def test_xml_array():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -103,9 +105,7 @@ def test_array():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
     schema.set_ns_prefix('tns', 'http://tests.python-zeep.org/')
 
     address_type = schema.get_element('tns:Address')
@@ -125,12 +125,11 @@ def test_array():
     """
     node = etree.Element('document', nsmap=schema._prefix_map_custom)
     address_type.render(node, obj)
-    print(etree.tostring(node))
     assert_nodes_equal(expected, node)
 
 
-def test_complex_type_unbounded_one():
-    node = etree.fromstring("""
+def test_xml_complex_type_unbounded_one():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -144,9 +143,7 @@ def test_complex_type_unbounded_one():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
     address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
     obj = address_type(foo=['foo'])
 
@@ -163,8 +160,8 @@ def test_complex_type_unbounded_one():
     assert_nodes_equal(expected, node)
 
 
-def test_complex_type_unbounded_named():
-    node = etree.fromstring("""
+def test_xml_complex_type_unbounded_named():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -177,9 +174,8 @@ def test_complex_type_unbounded_named():
             </sequence>
           </complexType>
         </schema>
-    """.strip())
+    """))
 
-    schema = xsd.Schema(node)
     address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
     obj = address_type()
     assert obj.foo == []
@@ -200,8 +196,8 @@ def test_complex_type_unbounded_named():
     assert_nodes_equal(expected, node)
 
 
-def test_complex_type_array_to_other_complex_object():
-    node = etree.fromstring("""
+def test_xml_complex_type_array_to_other_complex_object():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
           <xs:complexType name="Address">
@@ -216,9 +212,8 @@ def test_complex_type_array_to_other_complex_object():
           </xs:complexType>
           <xs:element name="ArrayOfAddress" type="ArrayOfAddress"/>
         </xs:schema>
-    """.strip())  # noqa
+    """))
 
-    schema = xsd.Schema(node)
     address_array = schema.get_element('ArrayOfAddress')
     obj = address_array()
     assert obj.Address == []
@@ -226,21 +221,26 @@ def test_complex_type_array_to_other_complex_object():
     obj.Address.append(schema.get_type('Address')(foo='foo'))
     obj.Address.append(schema.get_type('Address')(foo='bar'))
 
-    node = etree.fromstring("""
+    expected = """
         <?xml version="1.0"?>
-        <ArrayOfAddress>
-            <Address>
-                <foo>foo</foo>
-            </Address>
-            <Address>
-                <foo>bar</foo>
-            </Address>
-        </ArrayOfAddress>
-    """.strip())
+        <document>
+            <ArrayOfAddress>
+                <Address>
+                    <foo>foo</foo>
+                </Address>
+                <Address>
+                    <foo>bar</foo>
+                </Address>
+            </ArrayOfAddress>
+        </document>
+    """
+
+    result = render_node(address_array, obj)
+    assert_nodes_equal(expected, result)
 
 
-def test_complex_type_init_kwargs():
-    node = etree.fromstring("""
+def test_xml_complex_type_init_kwargs():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -255,9 +255,8 @@ def test_complex_type_init_kwargs():
             </complexType>
           </element>
         </schema>
-    """.strip())
+    """))
 
-    schema = xsd.Schema(node)
     address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
     obj = address_type(
         NameFirst='John', NameLast='Doe', Email='j.doe@example.com')
@@ -266,8 +265,8 @@ def test_complex_type_init_kwargs():
     assert obj.Email == 'j.doe@example.com'
 
 
-def test_complex_type_init_args():
-    node = etree.fromstring("""
+def test_xml_complex_type_init_args():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -282,9 +281,7 @@ def test_complex_type_init_args():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
     address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
     obj = address_type('John', 'Doe', 'j.doe@example.com')
     assert obj.NameFirst == 'John'
@@ -292,107 +289,9 @@ def test_complex_type_init_args():
     assert obj.Email == 'j.doe@example.com'
 
 
-def test_group():
-    node = etree.fromstring("""
-        <?xml version="1.0"?>
-        <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
-                   xmlns:tns="http://tests.python-zeep.org/"
-                   targetNamespace="http://tests.python-zeep.org/"
-                   elementFormDefault="qualified">
-
-          <xs:element name="Address">
-            <xs:complexType>
-              <xs:group ref="tns:Name" />
-            </xs:complexType>
-          </xs:element>
-
-          <xs:group name="Name">
-            <xs:sequence>
-              <xs:element name="first_name" type="xs:string" />
-              <xs:element name="last_name" type="xs:string" />
-            </xs:sequence>
-          </xs:group>
-
-        </xs:schema>
-    """.strip())
-    schema = xsd.Schema(node)
-    address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
-
-    obj = address_type(first_name='foo', last_name='bar')
-
-    node = etree.Element('document')
-    address_type.render(node, obj)
-    expected = """
-        <document>
-            <ns0:Address xmlns:ns0="http://tests.python-zeep.org/">
-                <ns0:first_name>foo</ns0:first_name>
-                <ns0:last_name>bar</ns0:last_name>
-            </ns0:Address>
-        </document>
-    """
-    assert_nodes_equal(expected, node)
-
-
-def test_group_for_type():
-    node = etree.fromstring("""
-        <?xml version="1.0"?>
-        <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"
-                   xmlns:tns="http://tests.python-zeep.org/"
-                   targetNamespace="http://tests.python-zeep.org/"
-                   elementFormDefault="unqualified">
-
-          <xs:element name="Address" type="tns:AddressType" />
-
-          <xs:complexType name="AddressType">
-            <xs:sequence>
-              <xs:group ref="tns:NameGroup"/>
-              <xs:group ref="tns:AddressGroup"/>
-            </xs:sequence>
-          </xs:complexType>
-
-          <xs:group name="NameGroup">
-            <xs:sequence>
-              <xs:element name="first_name" type="xs:string" />
-              <xs:element name="last_name" type="xs:string" />
-            </xs:sequence>
-          </xs:group>
-
-          <xs:group name="AddressGroup">
-            <xs:annotation>
-              <xs:documentation>blub</xs:documentation>
-            </xs:annotation>
-            <xs:sequence>
-              <xs:element name="city" type="xs:string" />
-              <xs:element name="country" type="xs:string" />
-            </xs:sequence>
-          </xs:group>
-        </xs:schema>
-    """.strip())
-    schema = xsd.Schema(node)
-    address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
-
-    obj = address_type(
-        first_name='foo', last_name='bar',
-        city='Utrecht', country='The Netherlands')
-
-    node = etree.Element('document')
-    address_type.render(node, obj)
-    expected = """
-        <document>
-            <ns0:Address xmlns:ns0="http://tests.python-zeep.org/">
-                <first_name>foo</first_name>
-                <last_name>bar</last_name>
-                <city>Utrecht</city>
-                <country>The Netherlands</country>
-            </ns0:Address>
-        </document>
-    """
-    assert_nodes_equal(expected, node)
-
-
-def test_element_ref_missing_namespace():
+def test_xml_element_ref_missing_namespace():
     # For buggy soap servers (#170)
-    node = etree.fromstring("""
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -406,9 +305,7 @@ def test_element_ref_missing_namespace():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
 
     custom_type = schema.get_element('{http://tests.python-zeep.org/}bar')
     input_xml = load_xml("""
@@ -420,8 +317,8 @@ def test_element_ref_missing_namespace():
     assert item.foo == 'bar'
 
 
-def test_element_ref():
-    node = etree.fromstring("""
+def test_xml_element_ref():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -436,9 +333,7 @@ def test_element_ref():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
 
     foo_type = schema.get_element('{http://tests.python-zeep.org/}foo')
     assert isinstance(foo_type.type, xsd.String)
@@ -459,8 +354,8 @@ def test_element_ref():
     assert_nodes_equal(expected, node)
 
 
-def test_element_ref_occurs():
-    node = etree.fromstring("""
+def test_xml_element_ref_occurs():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -476,9 +371,7 @@ def test_element_ref_occurs():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
 
     foo_type = schema.get_element('{http://tests.python-zeep.org/}foo')
     assert isinstance(foo_type.type, xsd.String)
@@ -499,8 +392,8 @@ def test_element_ref_occurs():
     assert_nodes_equal(expected, node)
 
 
-def test_unqualified():
-    node = etree.fromstring("""
+def test_xml_unqualified():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:xsd="http://www.w3.org/2001/XMLSchema"
@@ -516,9 +409,8 @@ def test_unqualified():
             </complexType>
           </element>
         </schema>
-    """.strip())
+    """))
 
-    schema = xsd.Schema(node)
     address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
     obj = address_type(foo='bar')
 
@@ -535,8 +427,8 @@ def test_unqualified():
     assert_nodes_equal(expected, node)
 
 
-def test_defaults():
-    node = etree.fromstring("""
+def test_xml_defaults():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <xsd:schema
                 xmlns:xsd="http://www.w3.org/2001/XMLSchema"
@@ -546,25 +438,40 @@ def test_defaults():
           <xsd:element name="container">
             <xsd:complexType>
               <xsd:sequence>
-                <xsd:element name="foo" type="xsd:string" default="hoi"/>
+                <xsd:element name="item_1" type="xsd:string" default="hoi"/>
+                <xsd:element name="item_2" type="xsd:string" default="hoi" minOccurs="0"/>
               </xsd:sequence>
-              <xsd:attribute name="bar" type="xsd:string" default="hoi"/>
+              <xsd:attribute name="attr_1" type="xsd:string" default="hoi"/>
             </xsd:complexType>
           </xsd:element>
         </xsd:schema>
-    """.strip())
+    """))
 
-    schema = xsd.Schema(node)
     container_type = schema.get_element(
         '{http://tests.python-zeep.org/}container')
     obj = container_type()
-    assert obj.foo == "hoi"
-    assert obj.bar == "hoi"
+    assert obj.item_1 == "hoi"
+    assert obj.item_2 is None
+    assert obj.attr_1 == "hoi"
 
     expected = """
       <document>
-        <ns0:container xmlns:ns0="http://tests.python-zeep.org/" bar="hoi">
-          <ns0:foo>hoi</ns0:foo>
+        <ns0:container xmlns:ns0="http://tests.python-zeep.org/" attr_1="hoi">
+          <ns0:item_1>hoi</ns0:item_1>
+        </ns0:container>
+      </document>
+    """
+    node = etree.Element('document')
+    container_type.render(node, obj)
+    assert_nodes_equal(expected, node)
+
+    obj.item_2 = 'ok'
+
+    expected = """
+      <document>
+        <ns0:container xmlns:ns0="http://tests.python-zeep.org/" attr_1="hoi">
+          <ns0:item_1>hoi</ns0:item_1>
+          <ns0:item_2>ok</ns0:item_2>
         </ns0:container>
       </document>
     """
@@ -573,8 +480,8 @@ def test_defaults():
     assert_nodes_equal(expected, node)
 
 
-def test_defaults_parse():
-    node = etree.fromstring("""
+def test_xml_defaults_parse_boolean():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <xsd:schema
                 xmlns:xsd="http://www.w3.org/2001/XMLSchema"
@@ -584,29 +491,65 @@ def test_defaults_parse():
           <xsd:element name="container">
             <xsd:complexType>
               <xsd:sequence>
-                <xsd:element name="foo" type="xsd:string" default="hoi" minOccurs="0"/>
+                <xsd:element name="foo" type="xsd:boolean" default="false"/>
               </xsd:sequence>
-              <xsd:attribute name="bar" type="xsd:string" default="hoi"/>
+              <xsd:attribute name="bar" type="xsd:boolean" default="0"/>
             </xsd:complexType>
           </xsd:element>
         </xsd:schema>
-    """.strip())
+    """))
+
+    container_type = schema.get_element(
+        '{http://tests.python-zeep.org/}container')
+    obj = container_type()
+    assert obj.foo == "false"
+    assert obj.bar == "0"
+
+    expected = """
+      <document>
+        <ns0:container xmlns:ns0="http://tests.python-zeep.org/" bar="false">
+          <ns0:foo>false</ns0:foo>
+        </ns0:container>
+      </document>
+    """
+    node = etree.Element('document')
+    container_type.render(node, obj)
+    assert_nodes_equal(expected, node)
+
+
+def test_xml_defaults_parse():
+    schema = xsd.Schema(load_xml("""
+        <?xml version="1.0"?>
+        <xsd:schema
+                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
+                xmlns:tns="http://tests.python-zeep.org/"
+                elementFormDefault="qualified"
+                targetNamespace="http://tests.python-zeep.org/">
+          <xsd:element name="container">
+            <xsd:complexType>
+              <xsd:sequence>
+                <xsd:element name="item_1" type="xsd:string" default="hoi" minOccurs="0"/>
+              </xsd:sequence>
+              <xsd:attribute name="attr_1" type="xsd:string" default="hoi"/>
+            </xsd:complexType>
+          </xsd:element>
+        </xsd:schema>
+    """))
 
-    schema = xsd.Schema(node)
     container_elm = schema.get_element(
         '{http://tests.python-zeep.org/}container')
 
     node = load_xml("""
         <ns0:container xmlns:ns0="http://tests.python-zeep.org/">
-          <ns0:foo>hoi</ns0:foo>
+          <ns0:item_1>hoi</ns0:item_1>
         </ns0:container>
     """)
     item = container_elm.parse(node, schema)
-    assert item.bar == 'hoi'
+    assert item.attr_1 == 'hoi'
 
 
-def test_init_with_dicts():
-    node = etree.fromstring("""
+def test_xml_init_with_dicts():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <xsd:schema
                 xmlns:xsd="http://www.w3.org/2001/XMLSchema"
@@ -637,9 +580,8 @@ def test_init_with_dicts():
             </xsd:sequence>
           </xsd:complexType>
         </xsd:schema>
-    """.strip())
+    """))
 
-    schema = xsd.Schema(node)
     address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
     obj = address_type(name='foo', container={'service': [{'name': 'foo'}]})
 
@@ -661,9 +603,8 @@ def test_init_with_dicts():
     assert_nodes_equal(expected, node)
 
 
-
-def test_sequence_in_sequence():
-    node = load_xml("""
+def test_xml_sequence_in_sequence():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema
                 xmlns="http://www.w3.org/2001/XMLSchema"
@@ -683,8 +624,7 @@ def test_sequence_in_sequence():
           </element>
           <element name="foobar" type="xsd:string"/>
         </schema>
-    """)
-    schema = xsd.Schema(node)
+    """))
     element = schema.get_element('ns0:container')
     value = element(item_1="foo", item_2="bar")
 
@@ -702,7 +642,7 @@ def test_sequence_in_sequence():
     assert_nodes_equal(expected, node)
 
 
-def test_sequence_in_sequence_many():
+def test_xml_sequence_in_sequence_many():
     node = load_xml("""
         <?xml version="1.0"?>
         <schema
@@ -752,8 +692,8 @@ def test_sequence_in_sequence_many():
     assert_nodes_equal(expected, node)
 
 
-def test_complex_type_empty():
-    node = etree.fromstring("""
+def test_xml_complex_type_empty():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -768,12 +708,10 @@ def test_complex_type_empty():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
 
     container_elm = schema.get_element('{http://tests.python-zeep.org/}container')
-    obj = container_elm()
+    obj = container_elm(something={})
 
     node = etree.Element('document')
     container_elm.render(node, obj)
@@ -789,7 +727,7 @@ def test_complex_type_empty():
     assert item.something is None
 
 
-def test_schema_as_payload():
+def test_xml_schema_as_payload():
     schema = xsd.Schema(load_xml("""
         <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -833,7 +771,7 @@ def test_schema_as_payload():
     assert value._value_1['item-2'] == 'value-2'
 
 
-def test_nill():
+def test_xml_nill():
     schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
@@ -867,8 +805,8 @@ def test_nill():
     assert_nodes_equal(expected, node)
 
 
-def test_empty_xmlns():
-    node = load_xml("""
+def test_xml_empty_xmlns():
+    schema = xsd.Schema(load_xml("""
         <?xml version="1.0"?>
         <schema xmlns="http://www.w3.org/2001/XMLSchema"
                 xmlns:tns="http://tests.python-zeep.org/"
@@ -884,9 +822,7 @@ def test_empty_xmlns():
             </complexType>
           </element>
         </schema>
-    """.strip())
-
-    schema = xsd.Schema(node)
+    """))
 
     container_elm = schema.get_element('{http://tests.python-zeep.org/}container')
     node = load_xml("""
@@ -902,3 +838,50 @@ def test_empty_xmlns():
     """)
     item = container_elm.parse(node, schema)
     assert item._value_1 == 'foo'
+
+
+def test_xml_keep_objects_intact():
+    schema = xsd.Schema(load_xml("""
+        <?xml version="1.0"?>
+        <xsd:schema
+                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
+                xmlns:tns="http://tests.python-zeep.org/"
+                attributeFormDefault="qualified"
+                elementFormDefault="qualified"
+                targetNamespace="http://tests.python-zeep.org/">
+          <xsd:element name="Address">
+            <xsd:complexType>
+              <xsd:sequence>
+                <xsd:element name="name" type="xsd:string"/>
+                <xsd:element minOccurs="0" name="optional" type="xsd:string"/>
+                <xsd:element name="container" nillable="true" type="tns:Container"/>
+              </xsd:sequence>
+            </xsd:complexType>
+          </xsd:element>
+
+          <xsd:complexType name="Container">
+            <xsd:sequence>
+              <xsd:element maxOccurs="unbounded" minOccurs="0" name="service"
+                           nillable="true" type="tns:ServiceRequestType"/>
+            </xsd:sequence>
+          </xsd:complexType>
+
+          <xsd:complexType name="ServiceRequestType">
+            <xsd:sequence>
+              <xsd:element name="name" type="xsd:string"/>
+            </xsd:sequence>
+          </xsd:complexType>
+        </xsd:schema>
+    """))
+
+    address_type = schema.get_element('{http://tests.python-zeep.org/}Address')
+    obj = address_type(name='foo', container={'service': [{'name': 'foo'}]})
+
+    org_obj = copy.deepcopy(obj)
+
+    node = etree.Element('document')
+    address_type.render(node, obj)
+
+    print(org_obj)
+    print(obj)
+    assert org_obj['container']['service'] == obj['container']['service']
